# SPDX-FileCopyrightText: Copyright © 2024 Idiap Research Institute <contact@idiap.ch>
# SPDX-FileContributor: Hatef OTROSHI <hatef.otroshi@idiap.ch>
# SPDX-License-Identifier: MIT
'''
Note: If you use this implementation, please cite the following paper:
- Hatef Otroshi Shahreza and Sébastien Marcel. "Generating Synthetic Face Recognition
  Datasets by Exploring Face Embedding Hypersphere", The Thirteenth International 
  Conference on Learning Representations, 2025.
'''
import argparse
parser = argparse.ArgumentParser(description='Extract Embeddings')
parser.add_argument('--path_dataset', metavar='<path_dataset>', type= str, default='path_dataset',
                    help='path_dataset')
parser.add_argument('--n_ids', metavar='<n_ids>', type= int, default='10000',
                    help='n_ids')
parser.add_argument('--path_save', metavar='<path_save>', type= str, default='./points',
                    help='path_save')

args = parser.parse_args()


import torch
import torch.optim as optim
from tqdm import tqdm
import math



import os,sys
from insightface.app import FaceAnalysis
from PIL import Image
import numpy as np
app = FaceAnalysis(name='antelopev2', root='./', 
                   providers=['CPUExecutionProvider'])
app.prepare(ctx_id=0, det_size=(160, 160))


def extract_emb(ref_path):
    ID = ref_path.split(os.sep)[-1]
    img = Image.open(ref_path)
    img = np.array(img)[:,:,::-1]
    faces = app.get(img)
    if not faces: 
        # no faces detected
        print(f"issue with {ID}")
        return None
        # return torch.randn(1, 512).cuda()[0]
    faces = sorted(faces, key=lambda x:(x['bbox'][2]-x['bbox'][0])*(x['bbox'][3]-x['bbox'][1]))[-1]  # select largest face (if more than one detected)
    id_emb = torch.tensor(faces['embedding'], dtype=torch.float16)[None].cuda()
    id_emb = id_emb/torch.norm(id_emb, dim=1, keepdim=True)   # normalize embedding
    return id_emb[0]

import os
import glob
from tqdm import tqdm
os.makedirs(args.path_save, exist_ok=True) 

all_refs = glob.glob(f'{args.path_dataset}/*/*.jpg')

extra_ids=1500#extra id is to ensure we have enough embeddings if face detection fails
if args.n_ids+extra_ids>len(all_refs):
    raise ValueError(f'There are {len(all_refs)} images which are not enogh for {args.n_ids} as n_ids.')

all_embeddings=[]
from multiprocessing import Pool, cpu_count
print('cpu_count',cpu_count())
with Pool(32) as p:
      all_embeddings = list(tqdm(p.imap(extract_emb, all_refs[:args.n_ids+extra_ids]), total=args.n_ids+extra_ids))
      
all_embeddings = [emb for emb in all_embeddings if emb is not None]

all_embeddings = all_embeddings[:args.n_ids]

os.makedirs(args.path_save, exist_ok=True) 
np.save(f'{args.path_save}/init_points_{len(all_embeddings)}.npy', torch.stack(all_embeddings).cpu().numpy())