# SPDX-FileCopyrightText: Copyright © 2024 Idiap Research Institute <contact@idiap.ch>
# SPDX-FileContributor: Hatef OTROSHI <hatef.otroshi@idiap.ch>
# SPDX-License-Identifier: MIT
'''
Note: If you use this implementation, please cite the following paper:
- Hatef Otroshi Shahreza and Sébastien Marcel. "Generating Synthetic Face Recognition
  Datasets by Exploring Face Embedding Hypersphere", The Thirteenth International 
  Conference on Learning Representations, 2025.
'''
import os,sys
import cv2
sys.path.append(os.getcwd())

import torch
import random
import numpy as np
from tqdm import tqdm
import math

seed=2021
random.seed(seed)
np.random.seed(seed)
torch.manual_seed(seed)
torch.backends.cudnn.deterministic = True
torch.backends.cudnn.benchmark = False

device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')
print("************ NOTE: The torch device is:", device)


#=================== StyleGAN
sys.path.append('./stylegan3')  # clone stylgan3 repository

import pickle
import torch_utils

path_stylegan = 'stylegan2-ffhq-512x512.pkl'
with open(path_stylegan, 'rb') as f:
    StyleGAN = pickle.load(f)['G_ema']
    # StyleGAN.to(device)
    StyleGAN.eval()
    StyleGAN_synthesis = StyleGAN.synthesis
    StyleGAN_mapping   = StyleGAN.mapping
    StyleGAN_synthesis.eval()
    StyleGAN_mapping.eval()
    StyleGAN_synthesis.to(device)
    StyleGAN_mapping.to(device)

z_dim_StyleGAN = StyleGAN.z_dim
from src.Crop import Crop_and_resize
#========================================================
batch_size=16
total_number_ids = int(50*1e3)

iterations_per_epoch_train = int (total_number_ids/batch_size)

os.makedirs(f'images/',exist_ok=True)

id_threshold_inter_classs = 0.90

n_id = 0
# for id_num in tqdm(range(total_number_ids)):
pbar=tqdm(total=total_number_ids,  position=0, leave=True)
while n_id< total_number_ids:
    
    with torch.no_grad():
        # Generate batch of latent vectors
        noise0 = torch.randn(batch_size, z_dim_StyleGAN, device=device)
        # generate w from noise(z)
        w0 = StyleGAN_mapping(z=noise0, c=None, truncation_psi=0.90).detach()
        # syntheise images from w
        img = StyleGAN_synthesis(w0).detach()
        # clamp values for generated images prior to feature extractor
        img = torch.clamp(img, min=-1, max=1)
        img = (img + 1) / 2.0 # range: (0,1)
        img_croped = Crop_and_resize(img)
                

        for k in range(batch_size):
            img_k = img[k]
                        
            n_id+=1
            # save image img_k as as first image of the (m+1)th subject
            
            im = (img_k.cpu().numpy().transpose(1,2,0))
            im = (im * 255).astype(int)
            folder='images/{0:0=2d}/'.format(n_id//1000)
            os.makedirs(folder,exist_ok=True)
            cv2.imwrite(f'{folder}/{n_id}.jpg',np.array([im[:,:,2],im[:,:,1],im[:,:,0]]).transpose(1,2,0))
            #print(f'{folder}/{n_id}.jpg')
            pbar.update(1)
            if n_id== total_number_ids:
                break